%% example2.m
% 
%--------------- Calculate set of level flight trim conditions ------------
%
% Script calculates a set of level flight trim condtions for different
% calibrated airspeeds and angles-of-attack.  Plots trim conditions as
% a function of specified target condition
%
% University of Minnesota 
% Aerospace Engineering and Mechanics 
% Copyright 2011 Regents of the University of Minnesota. 
% All rights reserved.
%
% SVN Info: $Id: example2.m 284 2011-03-03 15:07:19Z murch $

% -------------V_s plot----------------
% Trim to set of air speeds
speeds=23:-1:10;

% Allocate variables to plot
alpha=zeros(length(speeds),1);
elevator=zeros(length(speeds),1);
throttle=zeros(length(speeds),1);

% Compute trim points
fprintf('\nLevel Flight Trim\n  Trimming at V_s:\n');
for trimpt=[1:length(speeds)],
    fprintf(1,'     %3.2f,',speeds(trimpt));
    % specify trim target
    TrimCondition.target = struct('V_s',speeds(trimpt),'gamma',0);
    [TrimCondition,OperatingPoint] = trim_UAV(TrimCondition,AC,0,0);
    fprintf(1,'Residual=%5.2f\n',speeds(trimpt)-OperatingPoint.op_report.Outputs(1).y);
    % pull out trim alpha, ele, thr
    alpha(trimpt)   =OperatingPoint.op_report.Outputs(3).y*180/pi;
    elevator(trimpt)=TrimCondition.Inputs.elevator*180/pi;
    throttle(trimpt)=TrimCondition.Inputs.throttle*100;
end
fprintf(' Done\n');

% Make V_s trim point plot
figure(1),clf
subplot(121),
[ax,h1,h2]=plotyy(speeds,[elevator,alpha],speeds,throttle); grid on
set(ax,{'YColor'},{'blue';'red'});
set([h1;h2],{'Color'},{'blue';'blue';'red'});
set([h1;h2],{'Marker'},{'o';'s';'>'})
set([h1;h2],{'MarkerFaceColor'},{'blue';'none';'red'});
title('Trimmed level flight by V_s');
legend([h1;h2], { 'elevator','alpha','throttle'} );
set([h1;h2],{'Marker'},{'o';'s';'>'});
ylabel(ax(1),'Elevator(deg),  alpha(deg)');
ylabel(ax(2),'Throttle(%)');
xlabel('V_s (m/s)')


%----------AOA plot---------------

%% Trim to set of angles of attack
AOA=[0:2:20]/180*pi;

% Variables to plot
V_s=zeros(length(AOA),1);
elevator=zeros(length(AOA),1);
throttle=zeros(length(AOA),1);
fprintf('  Trimming at alpha:\n');

for trimpt=[1:length(AOA)],
    fprintf(1,'      %3.2f,',AOA(trimpt)*180/pi);
    % specify trim target
    TrimCondition.target = struct('alpha',AOA(trimpt),'gamma',0);
    [TrimCondition,OperatingPoint] = trim_UAV(TrimCondition,AC,0,0);
    fprintf(1,' Residual=%5.2f\n',(AOA(trimpt)-OperatingPoint.op_report.Outputs(3).y)*180/pi);
    % pull out trim V_s, ele, thr
    V_s(trimpt)     =OperatingPoint.op_report.Outputs(1).y;
    elevator(trimpt)=TrimCondition.Inputs.elevator*180/pi;
    throttle(trimpt)=TrimCondition.Inputs.throttle*100;
end
fprintf(' Done\n');

% Make alpha trim point plot
subplot(122),
[ax,h1,h2]=plotyy(AOA*180/pi,elevator,AOA*180/pi,[V_s,throttle]); grid on
set(ax,{'YColor'},{[0 0 1];[1 0 0]});
set([h1;h2],{'Color'},{'blue';'red';'red'});
set([h1;h2],{'Marker'},{'o';'s';'>'})
set([h1;h2],{'MarkerFaceColor'},{'blue';'none';'red'});
title('Trimmed level flight by AOA');
legend([h1;h2],{ 'elevator','V_s','throttle'});
ylabel(ax(1),'Elevator(deg)');
ylabel(ax(2),'V_s(m/s) and throttle(%)');
xlabel(ax(1),'alpha (deg)');


