/******************************************************************************
 * FILE: L_navmain.c
 * DESCRIPTION:
 *
 *
 *
 * SOURCE:
 * LAST REVISED: 8/31/06 Jung Soon Jang
 *
 * University of Minnesota 
 * Aerospace Engineering and Mechanics 
 *  2011 Regents of the University of Minnesota. All rights reserved.
 *
 * $Id: L_navmain.c 253 2011-01-20 22:01:35Z murch $
 ******************************************************************************/

#include "utils/navfunc.h"



//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//prototypes
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
extern double get_Time();

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
//error characteristics of navigation parameters
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#define	    Vcov    		0.04                        // (0.2 m/s)^2
#define     Pcov    		(5.0e-5*D2R)*(5.0e-5*D2R)   // (5.0e-5*d2r)
#define     Rew     		6.359058719353925e+006      //earth radius
#define     Rns     		6.386034030458164e+006      //earth radius
#define     NAV_NSECS		100000000			    	//10 Hz


static short gps_init_count=0;
static MATRIX nxs,nF,nG,nGd,nu;
static MATRIX nPn,nQn,nRn,nKn,nRinv;
static MATRIX dcm;
static MATRIX euler,nIden;
static MATRIX ntmp99,ntmpr,ntmp91,ntmpr91;
static MATRIX ntmp66,ntmp96,ntmp33;
extern int sPort0;



// navigation thread
void *navigation(void *thread_id)
{
	short  i=0;
	struct imu		imulocal;
	struct gps		gpslocal;
	double yd[6], tnow, Ndt;
	static double tprev=0;
	int rc;


	/*++++++++++++++++++++++++++++++++++++++++++++++++
	 *matrix creation for navigation computation
	 *++++++++++++++++++++++++++++++++++++++++++++++++*/
	nxs   = mat_creat(9,1,ZERO_MATRIX);   //state x=[lat lon alt ve vn vup bax bay baz]'
	nF    = mat_creat(9,9,ZERO_MATRIX);   //system matrix
	nG    = mat_creat(9,6,ZERO_MATRIX);	 //input matrix
	nGd   = mat_creat(9,6,ZERO_MATRIX);	 //input matrix in discrete time
	nKn   = mat_creat(9,6,ZERO_MATRIX);   //gain matrix
	nu    = mat_creat(6,1,ZERO_MATRIX); nu[5][0] = GRAVITY_NOM;
	dcm   = mat_creat(3,3,ZERO_MATRIX);   //directional cosine matrix
	ntmp33= mat_creat(3,3,ZERO_MATRIX);
	euler = mat_creat(3,1,UNDEFINED);     //euler angles
	nIden = mat_creat(9,9,UNIT_MATRIX);   //identity matrix
	nRinv = mat_creat(6,6,ZERO_MATRIX);
	ntmp99= mat_creat(9,9,ZERO_MATRIX);
	ntmpr = mat_creat(9,9,ZERO_MATRIX);
	ntmp91= mat_creat(9,1,ZERO_MATRIX);
	ntmpr91=mat_creat(9,1,ZERO_MATRIX);
	ntmp66 =mat_creat(6,6,ZERO_MATRIX);
	ntmp96 =mat_creat(9,6,ZERO_MATRIX);

	nPn    = mat_creat(9,9,UNIT_MATRIX);  //error covariance matrix
	nQn    = mat_creat(9,9,ZERO_MATRIX);  //process noise error matrix

	nQn[0][0] = 0;
	nQn[1][1] = nQn[0][0];
	nQn[2][2] = nQn[0][0];
	nQn[3][3] = 0.18*0.18;
	nQn[4][4] = nQn[3][3];
	nQn[5][5] = nQn[3][3];
	nQn[6][6] = 1.0e-8;
	nQn[7][7] = nQn[6][6];
	nQn[8][8] = nQn[6][6];

	nRn = mat_creat(6,6,ZERO_MATRIX);   //measurement noise error matrix
	nRn[0][0] = Pcov; nRn[1][1] = nRn[0][0]; nRn[2][2] = 2.0*2.0;
	nRn[3][3] = 0.01; nRn[4][4] = nRn[3][3]; nRn[5][5] = 0.02;


	navpacket.err_type = got_invalid;

	pthread_mutex_lock(&mutex_sPort0);
	sprintf(&msg[3],"NAV thread created");
	write(sPort0, msg, sizeof(msg));
	pthread_mutex_unlock(&mutex_sPort0);

	while (1)
	{
		sleep(1);

		pthread_mutex_lock(&mutex_gps);
		if (gpspacket.err_type == data_valid)
		{
			if (++gps_init_count > 20){
				pthread_mutex_unlock(&mutex_gps);
				break;
			}
		}
		pthread_mutex_unlock(&mutex_gps);


	}
	navpacket.err_type = data_valid;

	//initialize navigation state variables with GPS (Lat,Lon,Alt)
	pthread_mutex_lock(&mutex_gps);
	nxs[0][0] = gpspacket.lat*D2R;
	nxs[1][0] = gpspacket.lon*D2R;
	nxs[2][0] = gpspacket.alt;
	pthread_mutex_unlock(&mutex_gps);



	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++
	//start main-loop
	//++++++++++++++++++++++++++++++++++++++++++++++++++++++++

	while ( ofpMode != terminate )
	{
		pthread_mutex_lock(&mutex_gps);
		rc = pthread_cond_wait (&trigger_navigation, &mutex_gps);

		if (rc == 0){

			// compute time-elapsed 'dt'
			tnow = get_Time();
			Ndt = tnow - tprev;
			tprev = tnow;
			// compute thread jitter
			threadstat[2].jitter_nsec = ( Ndt - 1/(double)NAV_Hz ) * 1E09;
			if(Ndt==0) Ndt = 1/(double)NAV_Hz;


			//copy information to local variables
			pthread_mutex_lock(&mutex_imu);
			gpslocal = gpspacket;
			imulocal = imupacket;
			pthread_mutex_unlock(&mutex_imu);
			pthread_mutex_unlock(&mutex_gps);

			if( gpslocal.err_type == data_valid ){

				//======================= run navigation algorithm ==============

				//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
				//matrix initialization
				//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
				euler[0][0] = imulocal.psi;
				euler[1][0] = imulocal.the;
				euler[2][0] = imulocal.phi;

				//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
				//fill out F and G
				//+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
				EulerToDcm(euler,MAG_DEC,ntmp33);
				mat_scalMul(ntmp33,Ndt,dcm);

				for(i=0;i<9;i++) nF[i][i]=0;
				nF[0][3] = Ndt/(Rns + nxs[2][0]);
				nF[1][4] = Ndt/((Rew + nxs[2][0])*cos(nxs[0][0]));
				nF[2][5] =-Ndt;
				nF[3][6] = -dcm[0][0]; nF[3][7] = -dcm[0][1]; nF[3][8] = -dcm[0][2];
				nF[4][6] = -dcm[1][0]; nF[4][7] = -dcm[1][1]; nF[4][8] = -dcm[1][2];
				nF[5][6] = -dcm[2][0]; nF[5][7] = -dcm[2][1]; nF[5][8] = -dcm[2][2];

				nG[3][0] =  dcm[0][0]; nG[3][1] =  dcm[0][1]; nG[3][2] =  dcm[0][2]; nG[3][3] = Ndt;
				nG[4][0] =  dcm[1][0]; nG[4][1] =  dcm[1][1]; nG[4][2] =  dcm[1][2]; nG[4][4] = Ndt;
				nG[5][0] =  dcm[2][0]; nG[5][1] =  dcm[2][1]; nG[5][2] =  dcm[2][2]; nG[5][5] = Ndt;

				//discretization of G
				mat_scalMul(nF,0.5,ntmp99); for(i=0;i<9;i++) ntmp99[i][i]+=1;
				mat_mul(ntmp99,nG,nGd);
				//discretization of F
				for(i=0;i<9;i++) nF[i][i]+=1;

				//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
				//propagation of navigation equation
				//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
				nu[0][0] = (imulocal.ax);
				nu[1][0] = (imulocal.ay);
				nu[2][0] = (imulocal.az);

				//nxs = nF*nxs + nGd*nu
				mat_mul(nF,nxs,ntmp91);
				mat_mul(nGd,nu,ntmpr91);
				mat_add(ntmp91,ntmpr91,nxs);

				//error covriance propagation: P = Fd*P*Fd' + Q
				mat_mul(nF,nPn,ntmp99);
				mat_tran(nF,ntmpr);
				mat_mul(ntmp99,ntmpr,nPn);
				for(i=0;i<9;i++) nPn[i][i] += nQn[i][i];

				//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
				//update using GPS
				//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
				//			if(gpslocal.err_type == data_valid) {

				//gain matrix Kn = P*H'*(H*P*H' + R)^-1
				mat_subcopy(nPn, 6, 6, ntmp66);
				for(i=0;i<6;i++)
					ntmp66[i][i] += nRn[i][i];
				mat_inv(ntmp66,nRinv);
				mat_subcopy(nPn, 9, 6, ntmp96);
				mat_mul(ntmp96,nRinv,nKn);

				//error covariance matrix update
				//P = (I - K*H)*P
				mat_subcopy(nKn, 9, 6, ntmp99);
				for(i=1;i<9;i++) {
					ntmp99[i][6]=ntmp99[i][7]=ntmp99[i][8]=0;
				}
				mat_sub(nIden,ntmp99,ntmpr);
				mat_mul(ntmpr,nPn, ntmp99);
				mat_copy(ntmp99,nPn);

				//state update
				yd[0] = (gpslocal.lat*D2R - nxs[0][0]);
				yd[1] = (gpslocal.lon*D2R - nxs[1][0]);
				yd[2] = (gpslocal.alt     - nxs[2][0]);
				yd[3] = (gpslocal.vn      - nxs[3][0]);
				yd[4] = (gpslocal.ve      - nxs[4][0]);
				yd[5] = (gpslocal.vd      - nxs[5][0]);

				for( i=0;i<9;i++ ){
					nxs[i][0] += nKn[i][0]*yd[0]+nKn[i][1]*yd[1]+nKn[i][2]*yd[2] \
							+ nKn[i][3]*yd[3]+nKn[i][4]*yd[4]+nKn[i][5]*yd[5];
				}

				//			} //gps update

				// ==================== end nav algorithm =========================

				pthread_mutex_lock(&mutex_gps);
				pthread_mutex_lock(&mutex_nav);
				navpacket.lat = nxs[0][0]*R2D;
				navpacket.lon = nxs[1][0]*R2D;
				navpacket.alt = nxs[2][0];
				navpacket.vn  = nxs[3][0];
				navpacket.ve  = nxs[4][0];
				navpacket.vd  = nxs[5][0];
				navpacket.time= get_Time();
				pthread_mutex_unlock(&mutex_nav);
				pthread_mutex_unlock(&mutex_gps);
			}
			else {
				pthread_mutex_lock(&mutex_nav);
				navpacket.time = get_Time();
				navpacket.err_type = got_invalid;
				pthread_mutex_unlock(&mutex_nav);
			}
		}
	}


	//free memory space
	mat_free(nxs);
	mat_free(nF);
	mat_free(nG);
	mat_free(nPn);
	mat_free(nQn);
	mat_free(nRn);
	mat_free(nRinv);
	mat_free(nKn);
	mat_free(euler);
	mat_free(nIden);
	mat_free(ntmp99);
	mat_free(ntmp91);
	mat_free(ntmpr91);
	mat_free(ntmpr);
	mat_free(nGd);
	mat_free(nu);
	mat_free(ntmp66);
	mat_free(ntmp96);
	mat_free(ntmp33);

	sprintf(&msg[3],"NAV thread terminated");
	write(sPort0, msg, sizeof(msg));
	pthread_exit(NULL);
}
