/******************************************************************************
 * FILE: HoneywellPitot.c
 *
 * DESCRIPTION:
 * This is an Object Oriented C class header that is able to read the data of 
 * HSC*RR*001PD*A3 Honeywell differential pressure sensor.
 *
 * CREATED: 06-28-10: Kyle Dobitz
 * University of Minnesota 
 * Aerospace Engineering and Mechanics 
 *  2011 Regents of the University of Minnesota. All rights reserved.
 *
 * $Id: HoneywellPitot.h 253 2011-01-20 22:01:35Z murch $
 ******************************************************************************/

typedef struct honeywellpitot * HoneywellPitot;

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Public methods
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// samples the instantaneous differential pressure
cyg_uint32 HoneywellPitot_readPressure(HoneywellPitot);

// Old: returns the sampled differential pressure from +1 psi to -1 psi
// New: returns the sampled differential pressure from 6.89475729 KPa to -1 6.89475729 KPa
double HoneywellPitot_getPressure(HoneywellPitot);
double HoneywellPitot_getSpeed(HoneywellPitot);

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Constructor takes as input the i2c interface device, and the minimum and maximum pressure
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
HoneywellPitot new_HoneywellPitot(cyg_i2c_device *, double, double);

//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// Destructor
//++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
void delete_HoneywellPitot(HoneywellPitot);
